'use strict'

const test = require('tape')
const cp = require('child_process')
const path = require('path')

const proxy_js = path.join(__dirname, '..', 'proxy.js')

runTests()

function runTests () {
  const expectedFailureArgs = [
    [],
    ['--port'],
    ['--port=:'],
    ['--port=:bar'],
    ['--port=foo'],
    ['--port=foo:'],
    ['--port=foo:bar']
  ]

  expectedFailureArgs.forEach(function (args) {
    const testName = 'expected arg failure with: \'' + args.join(' ') + '\''
    test(testName, function (t) {
      expectFailedLaunch(t, args)
    })
  })
}

function expectFailedLaunch (t, args) {
  const execOpts = {
    cwd: '/',  // run out of / to ensure no .rc files
    timeout: 5000
  }
  const cmd = ['node', proxy_js].concat(args).join(' ')

  cp.exec(cmd, execOpts, function (err, stdout, stderr) {
    if (!err) return t.fail('expected command to fail')
    t.equal(err.signal, null, 'expected error signal to be null')
    t.equal(err.code, 1, 'expected error code to be 1')

    t.equal(stdout, '', 'expected stdout to be empty')
    t.notEqual(stderr, '', 'expected stderr to contain an error message')
    t.end()
  })
}
