/**
 * Process structure extraction & definition.
 */

var computeAppId = require('../compute-app-id')

module.exports = addProcess

/**
 * Extract process structure from process & host.
 * Mutates Application instance gathered from getApp.
 *
 * @param {Process} proc
 * @param {Host} host
 * @param {Function} getApp pass app name to retrieve Application instance
 * @return {Application}
 */

function addProcess (proc, host, getApp) {
  var meta = proc.meta
  var reply = proc.reply

  var application = getApp(meta.app, {
    id: computeAppId(meta.app),
    name: meta.app
  })

  // ignore last three 3 digits to get consistent start time
  const startTime = Math.round((Date.now() - reply.uptime * 1000) / 1000) * 1000

  application.processes.push({
    id: meta.id,
    app: meta.app,
    pid: meta.pid,
    uptime: reply.uptime,
    starttime: startTime,
    version: '',
    title: reply.title,
    async: {
      handles: reply.activeHandles || 0,
      streams: reply.activeStreams || 0,
      requests: reply.activeRequests || 0
    },

    memory: {
      rss: reply.rss,
      total: reply.heapTotal,
      used: reply.heapUsed
    },

    graphs: {},

    host: meta.hostname,
    snapshots: [],
    cpu: reply.cpu
  })

  return application
}
