var createGraph = require('../graph-cache')

module.exports = computeSparklines

function computeSparklines (application, hosts, pollInteval, duration) {
  var storeTime = (duration * 1000 * 60)

  application.processes.forEach(function eachProcess (proc) {
    var hostname = proc.host
    var host = hosts[hostname]
    var processGraph = createGraph({type: 'process', id: proc.id}, storeTime)
    var hostGraph = createGraph({type: 'host', id: hostname}, storeTime)

    if (!host.computedSparklines) {
      host.computedSparklines = true
      hostGraph.add('load1m', host.stats.load1m)
      hostGraph.add('memoryPercentageUsed', host.stats.memoryPercentageUsed)
    }

    var hostData = hostGraph.get('load1m')

    processGraph.add('rss', proc.memory.rss, hostData)
    processGraph.add('heapTotal', proc.memory.total, hostData)
    processGraph.add('heapUsed', proc.memory.used, hostData)
    processGraph.add('asyncHandles', proc.async.handles, hostData)
    processGraph.add('asyncRequests', proc.async.requests, hostData)
  })
}
