/**
 * Find & intersect host & process data.
 * Required as process state is gathered from multiple API calls to the N|Solid hub.
 */

var merge = require('merge')
var camelize = require('camelize')

module.exports = intersect

/**
 * Perform pairwise intersection on hosts or processeses, based on meta.id.
 * Basically anything two arrays that contain objects with the shape:
 * ```js
 * { meta : { id: "something" } }
 * ```
 *
 * @param {Array} a Array of Objects with meta.id fields
 * @param {Array} b Array of Objects with meta.id fields
 * @return {Object} Intersection of data in `a` & `b`
 */

function intersect (a, b) {
  var valid = {}
  var pending = {}
  for (var i = 0; i < b.length; i++) {
    pending[b[i].meta.id] = b[i]
  }
  for (var j = 0; j < a.length; j++) {
    var key = a[j].meta.id
    if (pending[key]) {
      valid[key] = camelize(merge.recursive(a[j], pending[key]))
    }
  }
  return valid
}
