"use strict"

module.exports = cli
var Client = require("nsolid-rpcclient")
const debug = require("util").debuglog("nsolid")

function cli(command, remoteSocket, options) {
  var client = Client(remoteSocket)

  if (options.timeout) {
    // convert to millis
    options.timeout = options.timeout * 1000
    debug("setting timeout to %sms", options.timeout)
  }

  // NOTE routeTo, `id` and `app` are for talking to the N|Solid Hub
  var out
  if (options.id != null || options.app != null) {
    var opts = {}
    if (options.id != null) {
      opts.id = options.id
      if (options.app == null) {
        // due to the namespacing in etcd we have to have an app to look up by id
        console.error("if 'id' is specified, must also specify 'app'")
        process.exit(1)
      }
    }
    if (options.app != null) {
      opts.app = options.app
    }
    var localTimeout
    if (options.timeout != null) {
      opts.timeout = options.timeout
      // we *know* we're routing, so add some extra time to prevent races
      localTimeout = opts.timeout + 2000
    }

    debug("routeTo", command, opts, localTimeout)
    out = client.routeTo(command, opts, options.data, localTimeout)
  }
  else {
    // either direct or broadcast via the Hub
    debug("send", command, opts, options.timeout)
    out = client.send(command, options.data, options.timeout)
  }

  return out
}
