"use strict";

module.exports = Client

var REQUEST_TIMEOUT = 10 * 1000 // 10 seconds

var http = require("http")
var url = require("url")
var path = require("path")
var encode = require("querystring").encode
var request = require("client-request")

var debug = require("util").debuglog("nsolid")

var PassThrough = require("stream").PassThrough

function Client(endpoint) {
  if (!(this instanceof Client)) {
    return new Client(endpoint)
  }
  this.endpoint = endpoint
}

function routeUrl(command, params) {
  if (params != null && Object.keys(params).length) {
    return command + "?" + encode(params)
  }
  return command
}

Client.prototype.routeTo = function routeTo(command, params, data, timeout) {
  return this.send(routeUrl(command, params), data, timeout)
}

Client.prototype.routeToBuffered = function routeToBuffered(command, params, data, callback, timeout) {
  var isGet = (data == null || data == "")
  timeout = timeout || REQUEST_TIMEOUT
  var opts = {
    uri: "http://" + this.endpoint + "/" + routeUrl(command, params),
    body: (!isGet) ? JSON.stringify(data) : null,
    timeout: timeout,
    method: (isGet) ? "GET" : "POST"
  }
  request(opts, callback)
}

Client.prototype.send = function send(command, data, timeout) {
  if (this.endpoint == null) {
    throw new Error("cannot send without an endpoint defined. use `sendTo`")
  }
  var uri = this.endpoint + "/" + command
  return _send(uri, data, timeout)
}

// does not need id or app name because those are only relevant to the proxy
Client.sendTo = function sendTo(endpoint, command, data, timeout) {
  var uri = endpoint + "/" + command
  return _send(uri, data, timeout)
}

Client.sendToBuffered = function sendToBuffered(endpoint, command, data, callback, timeout) {
  var isGet = (data == null || data == "")
  timeout = timeout || REQUEST_TIMEOUT
  var opts = {
    uri: "http://" + endpoint + "/" + command,
    body: (!isGet) ? JSON.stringify(data) : null,
    timeout: timeout,
    method: (isGet) ? "GET" : "POST"
  }
  request(opts, callback)
}

function _send(uri, data, timeout) {
  var isGet = (data == null || data == "")
  timeout = timeout || REQUEST_TIMEOUT
  // NOTE obviously not HTTPS-capable yet
  if (uri.indexOf("http://") != 0) {
    uri = "http://" + uri
  }
  var opts = url.parse(uri)
  opts.method = (isGet) ? "GET" : "POST"

  var sentError = false
  function sendError(err) {
    if (sentError) {
      return
    }
    sentError = true
    clearTimeout(totalTimeout)
    futureStream.emit("error", err)
  }

  var futureStream = new PassThrough()
  debug("sending %s", opts.href)
  var req = http.request(opts, function response(res) {
    if (res.statusCode >= 300) {
      var contents = []
      res.on("data", function (chunk) {
        contents.push(chunk)
      })
      res.on("error", sendError)
      res.on("end", function onEnd() {
        sendError(new Error(Buffer.concat(contents).toString()))
        clearTimeout(totalTimeout)
      })
    }
    else {
      res.pipe(futureStream)
      res.on("error", sendError)
      res.on("end", function onEnd() {
        clearTimeout(totalTimeout)
      })
    }
  })

  var totalTimeout = setTimeout(function onTimeout() {
    req.abort()
    var err = new Error("Client request timeout")
    sendError(err)
  }, timeout)

  // socket timeout
  req.setTimeout(timeout, function onTimeout() {
    req.abort()
    var err = new Error("Socket Timeout on RPC send")
    sendError(err)
  })

  req.on("error", sendError)
  if (!isGet) {
    req.write(JSON.stringify(data))
  }
  req.end()

  return futureStream
}
